package com.muyu.common.cache.config;
/**
 * 抽象缓存层
 * @ClassName CacheAbsBasic
 * @Description CacheAbsBasic:类的描述
 * @Date 2024/9/30 1:39
 * @author MingWei.Zong(微醺)
 */

import com.muyu.common.redis.service.RedisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;


public abstract class CacheAbsBasic<K,V> implements CacheBasic<K,V> {

    @Autowired
    private RedisService redisService;

    @Override
    public void put(K key, V value) {

        try {
            redisService.setCacheObject(encode(key), value,30L, TimeUnit.MINUTES);
        } catch (Exception e) {
            throw new RuntimeException("运行时异常，异常信息为：{}"+e.getMessage());
        }
    }

    @Override
    public V get(K key) {

        try {
            return redisService.getCacheObject(encode(key));
        } catch (Exception e) {
            throw new RuntimeException("运行时异常，异常信息为：{}"+e.getMessage());
        }
    }

    @Override
    public void remove(K key) {

        try {
            redisService.deleteObject(encode(key));
        } catch (Exception e) {
            throw new RuntimeException("运行时异常，异常信息为：{}"+e.getMessage());
        }
    }

    @Override
    public boolean hashKey(K key){
        Boolean b = false;

        try {
            b = redisService.hasKey(encode(key));
        } catch (Exception e) {
            throw new RuntimeException("运行时异常，异常信息为：{}"+e.getMessage());
        }
        return b;
    }

}
