package com.muyu.common.system.remote.factory;


import com.muyu.common.core.domain.Result;
import com.muyu.common.system.remote.RemoteEntService;
import com.muyu.common.system.remote.RemoteWarnService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cloud.openfeign.FallbackFactory;
import org.springframework.stereotype.Component;

/**
 * 预警服务降级处理--通过预警中车辆类型ID查询车辆信息
 * @ClassName RemoteWarnFallbackFactory
 * @PATH com.muyu.common.system.remote.factory:类的路径
 * @Date 2024/10/8 10:59
 * @author MingWei.Zong(微醺)
 */
@Component
public class RemoteWarnFallbackFactory implements FallbackFactory<RemoteWarnService> {
    private static final Logger log = LoggerFactory.getLogger(RemoteWarnFallbackFactory.class);

    @Override
    public RemoteWarnService create(Throwable throwable) {
        log.error("通过预警中车辆类型ID查询车辆信息服务调用失败:{}", throwable.getMessage());
        return new RemoteWarnService() {
            @Override
            public Result carManageShowByCarTypeId(Long carTypeId) {
                return Result.error("通过预警中车辆类型ID查询车辆信息失败:" + throwable.getMessage());
            }
        };
    }

}
