package com.muyu.enterprise.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import com.muyu.common.system.remote.RemoteEntService;
import com.muyu.enterprise.cache.CarManageCacheService;
import com.muyu.enterprise.domain.CarConfig;
import com.muyu.enterprise.domain.CarManage;
import com.muyu.enterprise.domain.car.Vehicle;
import com.muyu.enterprise.domain.dto.CarDTO;
import com.muyu.enterprise.domain.vo.CarVO;
import com.muyu.enterprise.mapper.CarManageMapper;
import com.muyu.enterprise.service.CarConfigService;
import com.muyu.enterprise.service.CarManageService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 车辆管理--业务实现层
 * @ClassName CarManageServiceImpl
 * @Description 车辆管理 ServiceImpl 层
 * @author  MingWei.Zong(微醺)
 * @Date 2024/9/28 16:52
 */
@Service
public class CarManageServiceImpl extends ServiceImpl<CarManageMapper, CarManage>
        implements CarManageService {

    @Autowired
    private CarConfigService carConfigService;
    @Autowired
    private CarManageCacheService carManageCacheService;
    @Resource
    private RemoteEntService remoteEntService;
    @Autowired
    private CarManageMapper carManageMapper;

    /**
     * 车辆列表2
     * @param carDTO
     * @return
     */
    @Override
    public List<CarVO> carListShow2(CarDTO carDTO) {
                // 获取 企业
        CarManage carManage = BeanUtil.copyProperties(carDTO, CarManage.class);

        List<CarManage> list = lambdaQuery()
                .eq(carManage.getCarVin() != null && carManage.getCarVin() != "", CarManage::getCarVin, carManage.getCarVin())
                .eq(carManage.getCarModel() != null && carManage.getCarModel() != "", CarManage::getCarModel, carManage.getCarModel())
                .eq(carManage.getCarBrand() != null && carManage.getCarBrand() != "", CarManage::getCarBrand, carManage.getCarBrand())
                .eq(carManage.getCarStatus() != null && carManage.getCarStatus() > 0, CarManage::getCarStatus, carManage.getCarStatus())
                .eq(carManage.getEnterpriseId() != null && carManage.getEnterpriseId() > 0, CarManage::getEnterpriseId, carManage.getEnterpriseId())
                .list();
//        List<CarManage> list = list();

        // 把 查询出来的车辆数据 copy 到 CarVO
        List<CarVO> carVOS = BeanUtil.copyToList(list, CarVO.class);
        // 遍历赋值
        carVOS.forEach(carVO -> {
            // 查询出对象,用于赋值
            CarConfig carConfig = carConfigService.selectByConfigId(carVO.getConfigId());
            carVO.setConfigName(carConfig.getConfigName());
            carVO.setEnergyType(carConfig.getEnergyType());
            carVO.setGearType(carConfig.getGearType());
            // 查询出对象,用于赋值   ----------远调企业
//            SysEnt data = remoteEntService.getById(carVO.getEnterpriseId()).getData();
//            carVO.setName(data.getName());
            // 存到 redis
            carManageCacheService.put(carVO.getCarVin(),carVOS);
        });
        return carVOS;
    }
    @Override
    public List<CarVO> CarListShow(CarDTO carDTO) {
        // 获取 企业
        CarManage carManage = BeanUtil.copyProperties(carDTO, CarManage.class);
        return carManageMapper.selectJoinList(CarVO.class, new MPJLambdaWrapper<CarManage>()
                .selectAll(CarManage.class) // 查询所有车辆表
                .select(CarConfig::getConfigName)
                .select(CarConfig::getGearType)
                .select(CarConfig::getEnergyType)
                .leftJoin(CarConfig.class, CarConfig::getConfigId, CarManage::getConfigId)
                .eq(carManage.getCarVin() != null && carManage.getCarVin() != "", CarManage::getCarVin, carManage.getCarVin())
                .eq(carManage.getCarModel() != null && carManage.getCarModel() != "", CarManage::getCarModel, carManage.getCarModel())
                .eq(carManage.getCarBrand() != null && carManage.getCarBrand() != "", CarManage::getCarBrand, carManage.getCarBrand())
                .eq(carManage.getCarStatus() != null && carManage.getCarStatus() > 0, CarManage::getCarStatus, carManage.getCarStatus())
                .eq(carManage.getEnterpriseId() != null && carManage.getEnterpriseId() > 0, CarManage::getEnterpriseId, carManage.getEnterpriseId())
        );


    }

    @Override
    public void insertCar(CarManage carVO) {
        CarVO carVO1 = BeanUtil.copyProperties(carVO, CarVO.class);
        List<CarVO> carVOS = new ArrayList<>();
        carVOS.add(carVO1);
        save(carVO);
        // 存到缓存中去
        carManageCacheService.put(carVO.getCarVin(),carVOS);
    }

    /**
     * 通过 carTypeId 查询车辆信息
     * @param carTypeId
     * @return
     */
    @Override
    public CarManage carManageShowByCarTypeId(Long carTypeId) {
        return getById(carTypeId);
    }

    /**
     * 通过id查询
     * @param carId
     * @return
     */
    @Override
    public CarManage carManageShowByCarId(Long carId) {
        return getById(carId);
    }
    /**
     * 通过车辆vin码查询模板id
     * @param carManageVin 请求对象
     * @return 返回结果
     */
    @Override
    public Long findByCarManageVin(String carManageVin) {
        Long templateId = carManageMapper.findByCarManageVin(carManageVin);
        return templateId;
    }

}
