package com.muyu.enterprise.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.enterprise.cache.CarWarnCacheService;
import com.muyu.enterprise.domain.CarManage;
import com.muyu.enterprise.domain.WarnRule;
import com.muyu.enterprise.domain.WarnStrategy;
import com.muyu.enterprise.domain.vo.WarnVo;
import com.muyu.enterprise.mapper.WarnStrategyMapper;
import com.muyu.enterprise.service.CarManageService;
import com.muyu.enterprise.service.IWarnRuleService;
import com.muyu.enterprise.service.IWarnStrategyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.List;

/**
 * 预警策略Service业务层处理
 *
 * @author muyu
 * @date 2024-09-20
 */
@Service
public class WarnStrategyServiceImpl
        extends ServiceImpl<WarnStrategyMapper, com.muyu.enterprise.domain.WarnStrategy>
        implements IWarnStrategyService {

    @Autowired
    private CarManageService carManageService;
    @Resource
    private IWarnRuleService warnRuleService;
    @Autowired
    private CarWarnCacheService carWarnCacheService;
    @Autowired
    private WarnStrategyMapper warnStrategyMapper;

    /**
     * 精确查询预警策略
     *
     * @param id 预警策略主键
     * @return 预警策略
     */
    @Override
    public com.muyu.enterprise.domain.WarnStrategy selectWarnStrategyById(Long id)
    {
        LambdaQueryWrapper<com.muyu.enterprise.domain.WarnStrategy> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(id, "id不可为空");
        queryWrapper.eq(com.muyu.enterprise.domain.WarnStrategy::getId, id);
        return this.getOne(queryWrapper);
    }


    /**
     * 查询预警策略列表
     *
     * @param warnStrategy 预警策略
     * @return 预警策略
     */
    @Override
    public List<com.muyu.enterprise.domain.WarnStrategy> selectWarnStrategyList(com.muyu.enterprise.domain.WarnStrategy warnStrategy)
    {
        LambdaQueryWrapper<com.muyu.enterprise.domain.WarnStrategy> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotNull(warnStrategy.getCarTypeId())){
            queryWrapper.eq(com.muyu.enterprise.domain.WarnStrategy::getCarTypeId, warnStrategy.getCarTypeId());
        }
        if (StringUtils.isNotNull(warnStrategy.getStrategyName())){
            queryWrapper.like(com.muyu.enterprise.domain.WarnStrategy::getStrategyName, warnStrategy.getStrategyName());
        }
        if (StringUtils.isNotNull(warnStrategy.getTemplateId())){
            queryWrapper.eq(com.muyu.enterprise.domain.WarnStrategy::getTemplateId, warnStrategy.getTemplateId());
        }
        // 1.获取 车辆 vin 码
        CarManage carManage = carManageService.carManageShowByCarTypeId(warnStrategy.getCarTypeId());
        List<WarnStrategy> list = list(queryWrapper);
        WarnVo warnVo = new WarnVo();
        // 遍历 赋值
        list.forEach(warnStrategy1 -> {
            // 2.查询 对应的策略有没有规则
            List<WarnRule> warnRules = warnRuleService.selectByStrategyId(warnStrategy1.getId());
            // 3.判断有没有对应的规则
            if(warnRules.size()!=0){// 有规则
                // 把查询出来的预警规则放入自定义的实体类
                warnVo.setWarnRuleList(warnRules);
            }
            warnVo.setId(warnStrategy1.getId());
            warnVo.setCarTypeId(warnStrategy1.getCarTypeId());
            warnVo.setStrategyName(warnStrategy1.getStrategyName());
            warnVo.setTemplateId(warnStrategy1.getTemplateId());
        });
        // 存储到 redis 中去
        carWarnCacheService.put(carManage.getCarVin(),warnVo);
        return list;
    }

    /**
    * 唯一 判断
    * @param warnStrategy 预警策略
     * @return 预警策略
    */
    @Override
    public Boolean checkIdUnique(com.muyu.enterprise.domain.WarnStrategy warnStrategy) {
        LambdaQueryWrapper<com.muyu.enterprise.domain.WarnStrategy> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(com.muyu.enterprise.domain.WarnStrategy::getId, warnStrategy.getId());
        return this.count(queryWrapper) > 0;
    }

    @Override
    public List<com.muyu.enterprise.domain.WarnStrategy> strategyList(com.muyu.enterprise.domain.WarnStrategy warnStrategy) {
        return  warnStrategyMapper.strategyList(warnStrategy);

    }

}
