package com.muyu.analysis.parsing.mqtt;

import com.alibaba.fastjson2.JSONObject;
import com.muyu.analysis.parsing.mqtt.service.MqttClientService;
import com.muyu.enterprise.cache.CarMessageValueCacheService;
import com.muyu.enterprise.cache.CarVehicleCacheService;
import com.muyu.enterprise.cache.CarVehicleTypeCacheService;
import com.muyu.enterprise.domain.MqttProperties;
import com.muyu.enterprise.domain.car.Vehicle;
import com.muyu.enterprise.domain.car.VehicleType;
import com.muyu.enterprise.domain.resp.car.MessageValueListResp;
import com.muyu.enterprise.remote.RemoteMessageValueService;
import com.muyu.enterprise.remote.RemoteVehicleService;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import com.alibaba.fastjson.JSON;
import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 协议解析处理数据发送传送到队列
 * @Author：李庆帅
 * @Package：com.muyu.analysis.parsing.MQTT
 * @Project：cloud-server
 * @name：ParsingMQTT
 * @Date：2024/9/29 16:08
 */
@Log4j2
@Component
public class ParsingMQTT {

    private final static String topic = "sysCar_vin_topic";


    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    @Resource
    private KafkaProducer<String, String> kafkaProducer;

    @Resource
    private RemoteVehicleService remoteVehicleService;

    @Resource
    private RemoteMessageValueService remoteMessageValueService;

    //车辆信息
    @Resource
    private  CarVehicleCacheService vehicleCacheService;

    //车辆类型信息
    @Resource
    private  CarVehicleTypeCacheService vehicleTypeCacheService;

    //报文模版信息
    @Resource
    private  CarMessageValueCacheService allMessageValueCacheService;

    @Autowired
    private MqttClientService mqttClientService;
    /**
     * 协议解析
     */
    @PostConstruct
    public JSONObject mqttClient(MqttMessage mqttMessage, String messageStr) {
        String payload = new String(mqttMessage.getPayload());
        log.info("====:{}", payload);
        /**
         * 协议解析
         * @param messageStr
         * @return
         */
            //根据空格切割数据
            String[] hexArray = messageStr.split(" ");
            StringBuilder result = new StringBuilder();
            //遍历十六进制数据转换为字符
            for (String hex : hexArray) {
                int decimal = Integer.parseInt(hex, 16);
                result.append((char) decimal);
            }
            //取出车辆VIN码
            String vehicleVin = result.substring(1, 18);
            log.info("车辆VIN码: " + vehicleVin);
            //根据车辆VIN码查询报文模板ID
            // 根据车辆VIN码查询车辆信息
            Vehicle vehicle = vehicleCacheService.get(vehicleVin);
            VehicleType vehicleType = vehicleTypeCacheService.get(String.valueOf(vehicle.getVehicleTypeId()));
            Long templateId = vehicleType.getMessageTemplateId();
            List<MessageValueListResp> templateList = allMessageValueCacheService.get(String.valueOf(templateId));
            //判断报文模板列表不为空
            if (templateList.isEmpty()) {
                log.info("报文模版为空");
                throw new RuntimeException("报文模版为空");
            }
            //存储报文模版解析后的数据
            JSONObject jsonObject = new JSONObject();
            for (MessageValueListResp messageValue : templateList) {
                //起始位下标
                Integer startIndex = messageValue.getMessageStartIndex() - 1;
                //结束位下标
                Integer endIndex = messageValue.getMessageEndIndex();
                //根据报文模版截取数据
                String value = result.substring(startIndex, endIndex);
                //存入数据
                jsonObject.put(messageValue.getMessageLabel(), value);
            }
            return jsonObject;
    }







}
