package com.muyu.analysis.parsing.consumer;


import com.muyu.analysis.parsing.mqtt.service.MqttClientService;
import com.muyu.enterprise.domain.MqttProperties;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;



/**
 * @Author：李庆帅
 * @Package：com.muyu.analysis.parsing.consumer
 * @Project：cloud-server
 * @name：RabbitListenerComponent
 * @Date：2024/10/10 20:24
 */

@Component
@Slf4j
public class RabbitListenerComponent {
    @Autowired
    private MqttClientService mqttClientService;

    private static final String FORM_QUEUE = "GO_LINE";

    @RabbitListener(queuesToDeclare = @Queue(value = FORM_QUEUE, durable = "true"))
    public void downline(MqttProperties mqttProperties, Message message, Channel channel) {
        try {
            mqttClientService.connectAndSubscribeAsync(mqttProperties);
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("处理RabbitMQ消息时发生错误", e);
        }
    }

}
