package com.muyu.vehiclegateway.config;

import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teautil.Common;
import com.aliyun.teautil.models.RuntimeOptions;
import com.muyu.common.redis.service.RedisService;
import com.muyu.vehiclegateway.aliyun.CreateClient;
import com.muyu.vehiclegateway.domain.Instance;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;


/**
 * @ClassName AddClientConfig
 * @Description 描述
 * @Author YiBo.Liu
 * @Date 2024/10/8 16:13
 */
@Log4j2
@Component
public class ClientConfig {

    @Autowired
    private static StringRedisTemplate redisTemplate;

    @Autowired
    private static RedisService redisService;

    public static String addServer() throws Exception {

        // 创建阿里云ECS客户端
        Client client = CreateClient.createClient();
        // 配置系统盘参数
        RunInstancesRequest.RunInstancesRequestSystemDisk systemDisk = new RunInstancesRequest.RunInstancesRequestSystemDisk()
                .setSize("40") // 设置系统盘大小为40GB
                .setCategory("cloud_essd"); // 设置系统盘类型为cloud_essd
        // 创建创建实例请求对象并设置参数
        RunInstancesRequest runInstancesRequest = new RunInstancesRequest()
                // 设置地域ID
                .setRegionId("cn-shanghai")
                // 设置镜像ID
                .setImageId("m-uf6f7atj16s3cjn9q5l8")
                // 设置实例类型
                .setInstanceType("ecs.t6-c1m1.large")
                // 设置安全组ID
                .setSecurityGroupId("sg-uf6f6dvazjqv127a4yyt")
                // 设置虚拟交换机ID
                .setVSwitchId("vsw-uf6htez4ox9k2c4cs8505")
                // 设置实例名称
                .setInstanceName("server-mqtt")
                // 设置实例付费类型为后付费按量付费
                .setInstanceChargeType("PostPaid")
                // 设置互联网最大出带宽为1 Mbps
                .setInternetMaxBandwidthOut(1)
                // 设置系统盘配置
                .setSystemDisk(systemDisk)
                // 设置主机名
                .setHostName("root")
                // 设置实例密码
                .setPassword("Six@211206")
                // 设置创建实例的数量
                .setAmount(1);



        // 创建运行时选项对象
        RuntimeOptions runtime = new RuntimeOptions();

        // 尝试执行创建实例请求
        try {

            RunInstancesResponse runInstancesResponse = client.runInstancesWithOptions(runInstancesRequest, runtime);
            RunInstancesResponseBody body = runInstancesResponse.getBody();
            RunInstancesResponseBody.RunInstancesResponseBodyInstanceIdSets instanceIdSets = body.getInstanceIdSets();

            List<Instance> instanceList = new ArrayList<>();
            List<String> instanceIps = new ArrayList<>();


            Thread.sleep(20000);

            DescribeInstancesResponse describeInstancesResponse = queryInstanceDetails(client);
            // 获取实例的ID、公网IP和状态

            String  ip = "";

            List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> bodyInstancesInstances = describeInstancesResponse.getBody().getInstances().getInstance();
            for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance : bodyInstancesInstances) {
                String id = instance.getInstanceId();
                String ipAddress = instance.getPublicIpAddress().getIpAddress().toString();
                String status = instance.getStatus();
                Instance instance1 = new Instance(id,ipAddress,status);
                instanceList.add(instance1);
                instanceIps.add(ipAddress);

                log.info("实例id为:{}",instance.getInstanceId());
                log.info("实例ip为:{}",instance.getPublicIpAddress().ipAddress.get(0));
                log.info("实例状态为{}",instance.getStatus());

                ip = instance.getPublicIpAddress().ipAddress.get(0);

                //每个 实例ip对应的绑定数量
                redisTemplate.opsForHash().put("serverBindings", ipAddress, 0);

            }

            log.info("====>创建的实例集合:"+instanceList);
            //把实例信息存入redis
            redisService.setCacheList("instanceList",instanceList);
            //实例ip存入redis方便做轮询
            redisService.setCacheList("ipList",instanceIps);

            return ip;
        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.error(error.getMessage());
            // 诊断地址
            log.error(error.getData().get("Recommend"));
            Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            log.error(error.getMessage());
            // 诊断地址
            log.error(error.getData().get("Recommend"));
            Common.assertAsString(error.message);
        }
            return null;
    }

    private static DescribeInstancesResponse queryInstanceDetails(Client client) {

        // 创建查询实例请求对象并设置参数
        DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest()
                .setInstanceName("server-mqtt")
                .setRegionId("cn-shanghai"); // 设置地域ID // 设置实例ID
        // 创建运行时选项对象
        RuntimeOptions runtime = new RuntimeOptions();

        // 尝试执行查询实例请求
        try {
            return client.describeInstancesWithOptions(describeInstancesRequest, runtime);
        } catch (TeaException e) {
            // 捕获特定的TeaException并打印详细信息
            log.info("TeaException occurred: " + e.getMessage());
            e.printStackTrace();
        } catch (Exception e) {
            // 捕获其他所有异常
            log.info("An error occurred: " + e.getMessage());
            e.printStackTrace();
        }
        return null;
    }


}
