package com.muyu.vehiclegateway.config;

import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Exchange;
import org.springframework.amqp.core.ExchangeBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


@Log4j2
@Configuration
public class RabbitmqConfig {
    // 日志
    private static final Logger logger = LoggerFactory.getLogger(RabbitmqConfig.class);

    /**
     * 车辆上线队列
     */
    public static final String GO_ONLINE_QUEUE = "GO_ONLINE";

    /**
     * 车辆下线队列
     */
    public static final String GO_OFFLINE_QUEUE = "GO_OFFLINE";

    /**
     * 协议解析队列
     */
    public static final String SEND_OFFLINE_QUEUE = "GO_LINE";


    /**
     * 车辆上线交换机
     */
    public static final String ONLINE_EXCHANGE = "ONLINE_EXCHANGE";

    /**
     * 车辆下线交换机
     */
    public static final String OFFLINE_EXCHANGE = "OFFLINE_EXCHANGE";

    /**
     * 协议解析交换机
     */
    public static final String LINE_EXCHANGE = "LINE_EXCHANGE";

    /**
     * 车辆下线路由key
     */
    public static final String GO_OFFLINE_ROUTING_KEY = "GO_OFFLINE";
    /**
     * 车辆上线路由key
     */
    public static final String GO_ONLINE_ROUTING_KEY = "GO_ONLINE";
    /**
     * 协议解析路由key
     */
    public static final String SEND_ONLINE_ROUTING_KEY = "GO_LINE";

    /**
     * 声明交换机,做持久化
     */
    @Bean(ONLINE_EXCHANGE)
    public Exchange exchangeTopicsInformON() {
        try {
            Exchange exchange = ExchangeBuilder.topicExchange(ONLINE_EXCHANGE).durable(true).build();
            log.info("创建的交换机为: {}", ONLINE_EXCHANGE);
            return exchange;
        } catch (Exception e) {
            log.error("创建该: {} 交换机失败", ONLINE_EXCHANGE, e);
            throw e;
        }
    }

    /**
     * 声明交换机,做持久化
     */
    @Bean(OFFLINE_EXCHANGE)
    public Exchange exchangeTopicsInformOFF() {
        try {
            Exchange exchange = ExchangeBuilder.topicExchange(OFFLINE_EXCHANGE).durable(true).build();
            log.info("创建的交换机为: {}", OFFLINE_EXCHANGE);
            return exchange;
        } catch (Exception e) {
            log.error("创建该: {} 交换机失败", OFFLINE_EXCHANGE, e);
            throw e;
        }
    }

    /**
     * 声明交换机,做持久化
     */
    @Bean(LINE_EXCHANGE)
    public Exchange exchangeTopicsInform() {
        try {
            Exchange exchange = ExchangeBuilder.topicExchange(LINE_EXCHANGE).durable(true).build();
            log.info("创建的交换机为: {}", LINE_EXCHANGE);
            return exchange;
        } catch (Exception e) {
            log.error("创建该: {} 交换机失败", LINE_EXCHANGE, e);
            throw e;
        }
    }

    // 声明QUEUE_INFORM_EMAIL队列
    @Bean(GO_OFFLINE_QUEUE)
    public Queue queueInformEmail() {
        try {
            Queue queue = new Queue(GO_OFFLINE_QUEUE);
            log.info("创建的队列为: {}", GO_OFFLINE_QUEUE);
            return queue;
        } catch (Exception e) {
            log.error("创建该: {} 队列失败", GO_OFFLINE_QUEUE, e);
            throw e;
        }
    }

    // 声明QUEUE_INFORM_SMS队列
    @Bean(GO_ONLINE_QUEUE)
    public Queue queueInformSms() {
        try {
            Queue queue = new Queue(GO_ONLINE_QUEUE);
            log.info("创建的队列为: {}", GO_ONLINE_QUEUE);
            return queue;
        } catch (Exception e) {
            log.error("创建该: {} 队列失败", GO_ONLINE_QUEUE, e);
            throw e;
        }
    }

    // 声明QUEUE_INFORM_SMS队列
    @Bean(SEND_OFFLINE_QUEUE)
    public Queue queueInformSend() {
        try {
            Queue queue = new Queue(SEND_OFFLINE_QUEUE);
            log.info("创建的队列为: {}", SEND_OFFLINE_QUEUE);
            return queue;
        } catch (Exception e) {
            log.error("创建该: {} 队列失败", SEND_OFFLINE_QUEUE, e);
            throw e;
        }
    }

    //ROUTINGKEY_EMAIL队列绑定交换机，指定routingKey
    @Bean
    public Binding bindingQueueInformEmail(@Qualifier(GO_OFFLINE_QUEUE) Queue queue,
                                              @Qualifier(OFFLINE_EXCHANGE) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(GO_OFFLINE_ROUTING_KEY).noargs();
    }
    //ROUTINGKEY_SMS队列绑定交换机，指定routingKey
    @Bean
    public Binding bindingRoutingKeySms(@Qualifier(GO_ONLINE_QUEUE) Queue queue,
                                          @Qualifier(ONLINE_EXCHANGE) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(GO_ONLINE_ROUTING_KEY).noargs();
    }

    //ROUTINGKEY_SMS队列绑定交换机，指定routingKey
    @Bean
    public Binding bindingRoutingKeySend(@Qualifier(SEND_OFFLINE_QUEUE) Queue queue,
                                        @Qualifier(LINE_EXCHANGE) Exchange exchange){
        return BindingBuilder.bind(queue).to(exchange).with(SEND_ONLINE_ROUTING_KEY).noargs();
    }
}
